@@ -1,5 +1,5 @@
-use clap::arg_enum;
 use serde::{Deserialize, Serialize};
+use std::path::PathBuf;
 use toml;
 use tui::style::Color;
 
@@ -20,6 +20,7 @@ impl Default for Randomize {
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 struct ASCII {
+    path: Option<PathBuf>,
     #[serde(with = "ColorDef")]
     color: Color,
 }
@@ -28,10 +29,21 @@ impl Default for ASCII {
     fn default() -> Self {
         ASCII {
             color: Color::Reset,
+            path: None,
         }
     }
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub enum Palette {
+    // Light color variants
+    Light,
+    // Dark color variants
+    Dark,
+    // Entire palette (16-colors)
+    Full,
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(remote = "Color")]
 enum ColorDef {
@@ -62,6 +74,12 @@ pub struct InnerMargin {
     y: u16,
 }
 
+impl Default for InnerMargin {
+    fn default() -> Self {
+        InnerMargin { x: 1, y: 0 }
+    }
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Block {
     title: String,
@@ -69,51 +87,27 @@ pub struct Block {
     inner_margin: InnerMargin,
 }
 
-impl InnerMargin {
-    fn new(a: u16, b: u16) -> Self {
-        InnerMargin { x: a, y: b }
-    }
-}
-
-impl Block {
+impl Default for Block {
     fn default() -> Self {
         Block {
             title: String::new(),
             visible: false,
-            inner_margin: InnerMargin::new(1, 0),
-        }
-    }
-
-    fn new(ti: &str, vi: bool) -> Self {
-        Block {
-            title: ti.to_string(),
-            visible: vi,
-            inner_margin: InnerMargin::new(1, 0),
+            inner_margin: InnerMargin::default(),
         }
     }
 }
 
-/// This struct stores the BarStyle to display when --bar or bar config option is used.
 #[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct BarStyle {
+pub struct Bar {
     pub glyph: String,
-    pub visible: bool,
     pub symbol_open: char,
     pub symbol_close: char,
+    pub visible: bool,
 }
 
-/// This stores predefined `BarStyle` variations.
-#[derive(Debug, Clone, Serialize, Deserialize)]
-pub enum BarStyles {
-    Squared,
-    Rounded,
-    Angled,
-    Hidden,
-}
-
-impl Default for BarStyle {
+impl Default for Bar {
     fn default() -> Self {
-        BarStyle {
+        Bar {
             glyph: String::new(),
             symbol_open: '(',
             symbol_close: ')',
@@ -122,243 +116,6 @@ impl Default for BarStyle {
     }
 }
 
-impl BarStyle {
-    fn new(style: BarStyles) -> Self {
-        match style {
-            BarStyles::Squared => BarStyle {
-                glyph: "■".to_owned(),
-                symbol_open: '[',
-                symbol_close: ']',
-                visible: true,
-            },
-            BarStyles::Rounded => BarStyle {
-                glyph: "●".to_owned(),
-                symbol_open: '(',
-                symbol_close: ')',
-                visible: true,
-            },
-            BarStyles::Angled => BarStyle {
-                glyph: "×".to_owned(),
-                symbol_open: '<',
-                symbol_close: '>',
-                visible: true,
-            },
-            BarStyles::Hidden => BarStyle {
-                glyph: "\0".to_owned(),
-                symbol_open: '\0',
-                symbol_close: '\0',
-                visible: true,
-            },
-        }
-    }
-
-    pub fn hide_delimiters(&self) -> BarStyle {
-        BarStyle {
-            glyph: self.glyph.to_owned(),
-            symbol_open: '\0',
-            symbol_close: '\0',
-            visible: true,
-        }
-    }
-}
-
-arg_enum! {
-    #[derive(Debug)]
-    pub enum Themes {
-    Hydrogen,
-    Helium,
-    Lithium,
-    Beryllium,
-    Boron,
-    }
-}
-
-/// This is the struct which defines predefined as well as custom themes.
-#[derive(Debug, Clone)]
-pub struct Theme {
-    bar: BarStyle,
-    color: Color,
-    separator_color: Color,
-    separator: String,
-    spacing: usize,
-    padding: usize,
-    randomize: Randomize,
-    custom_ascii: ASCII,
-    r#box: Block,
-    pub keys: Keys,
-}
-
-impl Default for Theme {
-    fn default() -> Theme {
-        Theme {
-            bar: BarStyle::new(BarStyles::Rounded),
-            color: Color::Red,
-            separator_color: Color::White,
-            separator: "-".to_owned(),
-            spacing: 2,
-            padding: 0,
-            r#box: Block::default(),
-            custom_ascii: ASCII::default(),
-            randomize: Randomize::default(),
-            keys: Keys::default(),
-        }
-    }
-}
-
-impl Theme {
-    pub fn new(theme: Themes) -> Self {
-        match theme {
-            Themes::Hydrogen => Theme {
-                bar: BarStyle::new(BarStyles::Rounded),
-                color: Color::Red,
-                separator_color: Color::White,
-                separator: "-".to_owned(),
-                spacing: 2,
-                padding: 0,
-                r#box: Block::new(" Hydrogen ", true),
-                custom_ascii: ASCII::default(),
-                randomize: Randomize::default(),
-                keys: Keys::default(),
-            },
-            Themes::Helium => Theme {
-                bar: BarStyle::new(BarStyles::Squared),
-                color: Color::Green,
-                separator_color: Color::White,
-                separator: "=>".to_owned(),
-                spacing: 2,
-                padding: 0,
-                r#box: Block::new(" Helium ", false),
-                custom_ascii: ASCII::default(),
-                randomize: Randomize::default(),
-                keys: Keys::default(),
-            },
-            Themes::Lithium => Theme {
-                bar: BarStyle::new(BarStyles::Angled),
-                color: Color::Magenta,
-                separator_color: Color::White,
-                separator: "~".to_owned(),
-                spacing: 2,
-                padding: 0,
-                r#box: Block::new(" Lithium ", false),
-                custom_ascii: ASCII::default(),
-                randomize: Randomize::default(),
-                keys: Keys::default(),
-            },
-            Themes::Beryllium => Theme {
-                bar: BarStyle::new(BarStyles::Rounded),
-                color: Color::Yellow,
-                separator_color: Color::White,
-                separator: "->".to_owned(),
-                spacing: 2,
-                padding: 0,
-                r#box: Block::new(" Beryllium ", true),
-                custom_ascii: ASCII::default(),
-                randomize: Randomize::default(),
-                keys: Keys::default(),
-            },
-            Themes::Boron => Theme {
-                bar: BarStyle::new(BarStyles::Rounded),
-                color: Color::Blue,
-                separator_color: Color::White,
-                separator: "•".to_owned(),
-                spacing: 2,
-                padding: 0,
-                r#box: Block::new(" Boron ", false),
-                custom_ascii: ASCII::default(),
-                randomize: Randomize::default(),
-                keys: Keys::default(),
-            },
-        }
-    }
-    pub fn get_bar_style(&self) -> &BarStyle {
-        &self.bar
-    }
-
-    pub fn set_bar_style(&mut self, new_bar: BarStyle) {
-        self.bar = new_bar
-    }
-
-    pub fn get_separator(&self) -> &str {
-        &self.separator
-    }
-
-    pub fn set_separator(&mut self, separator: impl ToString) {
-        self.separator = separator.to_string()
-    }
-
-    pub fn get_separator_color(&self) -> Color {
-        self.separator_color
-    }
-
-    pub fn set_separator_color(&mut self, color: Color) {
-        self.separator_color = color
-    }
-
-    pub fn get_color(&self) -> Color {
-        self.color
-    }
-
-    pub fn set_color(&mut self, color: Color) {
-        self.color = color
-    }
-
-    pub fn get_padding(&self) -> usize {
-        self.padding
-    }
-
-    pub fn get_box_title(&self) -> String {
-        self.r#box.title.to_owned()
-    }
-
-    pub fn is_box_visible(&self) -> bool {
-        self.r#box.visible
-    }
-
-    pub fn is_key_color_randomized(&self) -> bool {
-        self.randomize.key_color
-    }
-
-    pub fn is_separator_color_randomized(&self) -> bool {
-        self.randomize.separator_color
-    }
-
-    pub fn get_horizontal_margin(&self) -> u16 {
-        self.r#box.inner_margin.x
-    }
-
-    pub fn get_vertical_margin(&self) -> u16 {
-        self.r#box.inner_margin.y
-    }
-
-    pub fn get_custom_ascii_color(&self) -> Color {
-        self.custom_ascii.color
-    }
-
-    pub fn using_custom_ascii_color(&self) -> bool {
-        if self.custom_ascii.color == Color::Reset {
-            return false;
-        }
-
-        true
-    }
-
-    pub fn using_bars(&self) -> bool {
-        self.bar.visible
-    }
-
-    pub fn set_padding(&mut self, size: usize) {
-        self.padding = size
-    }
-
-    pub fn get_spacing(&self) -> usize {
-        self.spacing
-    }
-
-    pub fn set_spacing(&mut self, spacing: usize) {
-        self.spacing = spacing;
-    }
-}
-
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Keys {
     pub host: String,
@@ -406,74 +163,195 @@ impl Default for Keys {
     }
 }
 
-impl From<CustomTheme> for Theme {
-    fn from(custom: CustomTheme) -> Self {
+/// This structure defines the skeleton of custom themes which are deserialized from TOML files.
+/// See [https://github.com/Macchina-CLI/macchina/blob/main/theme/Carbon.toml](this) for an example
+/// theme.
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(default)]
+pub struct Theme {
+    custom_ascii: ASCII,
+    bar: Bar,
+    r#box: Block,
+    separator: String,
+    randomize: Randomize,
+    spacing: usize,
+    padding: usize,
+    palette: Option<Palette>,
+    hide_ascii: bool,
+    prefer_small_ascii: bool,
+    hide_bar_delimiters: bool,
+    pub keys: Keys,
+    #[serde(with = "ColorDef")]
+    key_color: Color,
+    #[serde(with = "ColorDef")]
+    separator_color: Color,
+}
+
+impl Default for Theme {
+    fn default() -> Self {
+        Theme {
+            key_color: Color::Blue,
+            separator_color: Color::Yellow,
+            separator: String::from("-"),
+            hide_ascii: false,
+            hide_bar_delimiters: false,
+            prefer_small_ascii: false,
+            palette: None,
+            spacing: 2,
+            padding: 2,
+            randomize: Randomize::default(),
+            custom_ascii: ASCII::default(),
+            bar: Bar::default(),
+            r#box: Block::default(),
+            keys: Keys::default(),
+        }
+    }
+}
+
+impl Theme {
+    pub fn new(custom: Theme) -> Self {
         Self {
             bar: custom.bar,
-            color: custom.color,
+            key_color: custom.key_color,
             separator: custom.separator,
             separator_color: custom.separator_color,
             spacing: custom.spacing,
             padding: custom.padding,
+            palette: custom.palette,
+            hide_bar_delimiters: custom.hide_bar_delimiters,
+            hide_ascii: custom.hide_ascii,
+            prefer_small_ascii: custom.prefer_small_ascii,
             r#box: custom.r#box,
             custom_ascii: custom.custom_ascii,
             randomize: custom.randomize,
             keys: custom.keys,
         }
     }
-}
 
-/// This structure defines the skeleton of custom themes which are deserialized from TOML files.
-/// See [https://github.com/Macchina-CLI/macchina/blob/main/theme/Carbon.toml](this) for an example
-/// theme.
-#[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(default)]
-pub struct CustomTheme {
-    #[serde(with = "ColorDef")]
-    color: Color,
-    #[serde(with = "ColorDef")]
-    separator_color: Color,
-
-    custom_ascii: ASCII,
-    bar: BarStyle,
-    r#box: Block,
-    separator: String,
-    randomize: Randomize,
-    spacing: usize,
-    padding: usize,
-    keys: Keys,
-}
-
-impl Default for CustomTheme {
-    fn default() -> Self {
-        Self {
-            bar: BarStyle::default(),
-            color: Color::Red,
-            separator: "->".to_string(),
-            separator_color: Color::White,
-            spacing: 0,
-            padding: 2,
-            r#box: Block::new("", false),
-            custom_ascii: ASCII::default(),
-            randomize: Randomize::default(),
-            keys: Keys::default(),
-        }
+    pub fn get_bar_style(&self) -> &Bar {
+        &self.bar
     }
-}
 
-impl CustomTheme {
-    /// Reads custom themes from $XDG_CONFIG_HOME/macchina/themes/{name}.toml
-    pub fn get_theme(name: &str) -> Result<Self, std::io::Error> {
+    pub fn set_bar_style(&mut self, new_bar: Bar) {
+        self.bar = new_bar
+    }
+
+    pub fn get_separator(&self) -> &str {
+        &self.separator
+    }
+
+    pub fn set_separator(&mut self, separator: impl ToString) {
+        self.separator = separator.to_string()
+    }
+
+    pub fn get_separator_color(&self) -> Color {
+        self.separator_color
+    }
+
+    pub fn set_separator_color(&mut self, color: Color) {
+        self.separator_color = color
+    }
+
+    pub fn get_key_color(&self) -> Color {
+        self.key_color
+    }
+
+    pub fn set_key_color(&mut self, color: Color) {
+        self.key_color = color
+    }
+
+    pub fn get_box_title(&self) -> String {
+        self.r#box.title.to_owned()
+    }
+
+    pub fn is_box_visible(&self) -> bool {
+        self.r#box.visible
+    }
+
+    pub fn is_key_color_randomized(&self) -> bool {
+        self.randomize.key_color
+    }
+
+    pub fn is_separator_color_randomized(&self) -> bool {
+        self.randomize.separator_color
+    }
+
+    pub fn get_horizontal_margin(&self) -> u16 {
+        self.r#box.inner_margin.x
+    }
+
+    pub fn get_vertical_margin(&self) -> u16 {
+        self.r#box.inner_margin.y
+    }
+
+    pub fn prefers_small_ascii(&self) -> bool {
+        self.prefer_small_ascii
+    }
+
+    pub fn is_ascii_hidden(&self) -> bool {
+        self.hide_ascii
+    }
+
+    pub fn get_custom_ascii_color(&self) -> Color {
+        self.custom_ascii.color
+    }
+
+    pub fn get_palette_type(&self) -> Option<&Palette> {
+        self.palette.as_ref()
+    }
+
+    pub fn using_custom_ascii_color(&self) -> bool {
+        if self.custom_ascii.color == Color::Reset {
+            return false;
+        }
+
+        true
+    }
+
+    pub fn get_custom_ascii_path(&self) -> Option<&PathBuf> {
+        self.custom_ascii.path.as_ref()
+    }
+
+    pub fn is_using_bars(&self) -> bool {
+        self.bar.visible
+    }
+
+    pub fn are_bar_delimiters_hidden(&self) -> bool {
+        self.hide_bar_delimiters
+    }
+
+    pub fn get_padding(&self) -> usize {
+        self.padding
+    }
+
+    pub fn set_padding(&mut self, size: usize) {
+        self.padding = size
+    }
+
+    pub fn get_spacing(&self) -> usize {
+        self.spacing
+    }
+
+    pub fn set_spacing(&mut self, spacing: usize) {
+        self.spacing = spacing;
+    }
+
+    pub fn hide_bar_delimiters(&mut self) {
+        self.bar.symbol_open = '\0';
+        self.bar.symbol_close = '\0';
+    }
+
+    /// Searches for and returns a theme from `~/.config/macchina/themes`
+    pub fn get_theme(name: &str, dir: Option<PathBuf>) -> Result<Self, std::io::Error> {
         use std::io::Read;
-        // check if theme exists in ~/.config/macchina/themes/{name}.toml
-        // TODO: look at more data paths?
         let mut theme_path = std::path::PathBuf::new();
-        theme_path.push(dirs::config_dir().ok_or_else(|| {
+        theme_path.push(dir.ok_or_else(|| {
             std::io::Error::new(
                 std::io::ErrorKind::NotFound,
                 "$XDG_CONFIG_HOME was not found; fallback $HOME/.config also failed.",
             )
         })?);
+
         theme_path.push(std::path::Path::new(&format!(
             "macchina/themes/{}.toml",
             name
@@ -488,21 +366,28 @@ impl CustomTheme {
         })
     }
 
-    // private function to print a custom theme for testing
-    fn __print_theme_test() {
-        let cust = CustomTheme {
-            bar: BarStyle::new(BarStyles::Squared),
-            separator: "=====>".to_string(),
+    fn _print_theme_test() {
+        let cust = Theme {
+            bar: Bar::default(),
+            separator: String::from("<-->"),
             spacing: 2,
             padding: 0,
-            r#box: Block::new("SomeTitle", true),
+            r#box: Block {
+                title: String::from("Neon"),
+                visible: true,
+                inner_margin: InnerMargin::default(),
+            },
             randomize: Randomize::default(),
-
-            color: Color::Rgb(10, 33, 51),
+            hide_ascii: false,
+            hide_bar_delimiters: false,
+            prefer_small_ascii: false,
+            palette: Some(Palette::Full),
+            key_color: Color::Rgb(10, 33, 51),
             custom_ascii: ASCII::default(),
             separator_color: Color::Indexed(100),
             keys: Keys::default(),
         };
+
         println!("{}", toml::to_string_pretty(&cust).unwrap());
     }
 }
